@@ -17,7 +17,11 @@
 #include "Python.h"
 #include "structmember.h"
 #include <exception>
+#if defined(_MSC_VER) && _MSC_VER == 1500
+typedef signed __int32    int32_t;
+#else
 #include <stdint.h>
+#endif
 
 
 #if PY_MAJOR_VERSION >= 3
@@ -35,6 +39,69 @@
 
 template class AnnoyIndexInterface<int32_t, float>;
 
+class HammingWrapper : public AnnoyIndexInterface<int32_t, float> {
+  // Wrapper class for Hamming distance, using composition.
+  // This translates binary (float) vectors into packed uint64_t vectors.
+  // This is questionable from a performance point of view. Should reconsider this solution.
+private:
+  int32_t _f_external, _f_internal;
+  AnnoyIndex<int32_t, uint64_t, Hamming, Kiss64Random> _index;
+  void _pack(const float* src, uint64_t* dst) {
+    for (int32_t i = 0; i < _f_internal; i++) {
+      dst[i] = 0;
+      for (int32_t j = 0; j < 64 && i*64+j < _f_external; j++) {
+	dst[i] |= (uint64_t)(src[i * 64 + j] > 0.5) << j;
+      }
+    }
+  };
+  void _unpack(const uint64_t* src, float* dst) {
+    for (int32_t i = 0; i < _f_external; i++) {
+      dst[i] = (src[i / 64] >> (i % 64)) & 1;
+    }
+  };
+public:
+  HammingWrapper(int f) : _f_external(f), _f_internal((f + 63) / 64), _index((f + 63) / 64) {};
+  void add_item(int32_t item, const float* w) {
+    vector<uint64_t> w_internal(_f_internal, 0);
+    _pack(w, &w_internal[0]);
+    _index.add_item(item, &w_internal[0]);
+  };
+  void build(int q) { _index.build(q); };
+  void unbuild() { _index.unbuild(); };
+  bool save(const char* filename) { return _index.save(filename); };
+  void unload() { _index.unload(); };
+  bool load(const char* filename) { return _index.load(filename); };
+  float get_distance(int32_t i, int32_t j) { return _index.get_distance(i, j); };
+  void get_nns_by_item(int32_t item, size_t n, size_t search_k, vector<int32_t>* result, vector<float>* distances) {
+    if (distances) {
+      vector<uint64_t> distances_internal;
+      _index.get_nns_by_item(item, n, search_k, result, &distances_internal);
+      distances->insert(distances->begin(), distances_internal.begin(), distances_internal.end());
+    } else {
+      _index.get_nns_by_item(item, n, search_k, result, NULL);
+    }
+  };
+  void get_nns_by_vector(const float* w, size_t n, size_t search_k, vector<int32_t>* result, vector<float>* distances) {
+    vector<uint64_t> w_internal(_f_internal, 0);
+    _pack(w, &w_internal[0]);
+    if (distances) {
+      vector<uint64_t> distances_internal;
+      _index.get_nns_by_vector(&w_internal[0], n, search_k, result, &distances_internal);
+      distances->insert(distances->begin(), distances_internal.begin(), distances_internal.end());
+    } else {
+      _index.get_nns_by_vector(&w_internal[0], n, search_k, result, NULL);
+    }
+  };
+  int32_t get_n_items() { return _index.get_n_items(); };
+  void verbose(bool v) { _index.verbose(v); };
+  void get_item(int32_t item, float* v) {
+    vector<uint64_t> v_internal(_f_internal, 0);
+    _index.get_item(item, &v_internal[0]);
+    _unpack(&v_internal[0], v);
+  };
+  void set_seed(int q) { _index.set_seed(q); };
+};
+
 // annoy python object
 typedef struct {
   PyObject_HEAD
@@ -60,6 +127,8 @@ py_an_new(PyTypeObject *type, PyObject *args, PyObject *kwargs) {
     self->ptr = new AnnoyIndex<int32_t, float, Euclidean, Kiss64Random>(self->f);
   } else if (!strcmp(metric, "manhattan")) {
     self->ptr = new AnnoyIndex<int32_t, float, Manhattan, Kiss64Random>(self->f);
+  } else if (!strcmp(metric, "hamming")) {
+    self->ptr = new HammingWrapper(self->f);
   } else {
     PyErr_SetString(PyExc_ValueError, "No such metric");
     return NULL;
@@ -73,16 +142,17 @@ static int
 py_an_init(py_annoy *self, PyObject *args, PyObject *kwargs) {
   // Seems to be needed for Python 3
   const char *metric = NULL;
-  PyArg_ParseTuple(args, "i|s", &self->f, &metric);
+  int f;
+  static char const * kwlist[] = {"f", "metric", NULL};
+  if (!PyArg_ParseTupleAndKeywords(args, kwargs, "i|s", (char**)kwlist, &f, &metric))
+    return NULL;
   return 0;
 }
 
 
 static void 
 py_an_dealloc(py_annoy* self) {
-  if (self->ptr) {
-    delete self->ptr;
-  }
+  delete self->ptr;
   Py_TYPE(self)->tp_free((PyObject*)self);
 }
 
